@@ -2,14 +2,16 @@ import keyword
 import re
 from abc import ABC, abstractmethod
 from collections import defaultdict
+from functools import lru_cache
 from pathlib import Path
-from typing import Any, DefaultDict, Dict, List, Optional, Set
+from typing import Any, DefaultDict, Dict, Iterator, List, Optional, Set
 
 from jinja2 import Environment, FileSystemLoader, Template
 from pydantic import BaseModel, root_validator
 
 from datamodel_code_generator import cached_property
 from datamodel_code_generator.imports import IMPORT_OPTIONAL, Import
+from datamodel_code_generator.reference import Reference
 from datamodel_code_generator.types import DataType
 
 TEMPLATE_DIR: Path = Path(__file__).parents[0] / 'template'
@@ -35,19 +37,28 @@ class DataModelFieldBase(BaseModel):
     data_type: DataType
     constraints: Any = None
     strip_default_none: bool = False
+    nullable: Optional[bool] = None
 
     @property
     def type_hint(self) -> str:
         type_hint = self.data_type.type_hint
-        if self.required:
-            return type_hint
-        if type_hint is None or type_hint == '':
+
+        if not type_hint:
             return OPTIONAL
+        elif self.nullable is not None:
+            if self.nullable:
+                return f'{OPTIONAL}[{type_hint}]'
+            return type_hint
+        elif self.required:
+            return type_hint
         return f'{OPTIONAL}[{type_hint}]'
 
     @property
     def imports(self) -> List[Import]:
-        if not self.required:
+        if self.nullable is None:
+            if not self.required:
+                return self.data_type.imports_ + [IMPORT_OPTIONAL]
+        elif self.nullable:
             return self.data_type.imports_ + [IMPORT_OPTIONAL]
         return self.data_type.imports_
 
@@ -66,16 +77,16 @@ class DataModelFieldBase(BaseModel):
 
     @root_validator
     def validate_root(cls, values: Any) -> Dict[str, Any]:
-        name = values.get('name')
+        name: Optional[str] = values.get('name')
         if name:
             if keyword.iskeyword(name):
                 alias = name
                 name += '_'
-            elif re.search(r'\W', name):  # pragma: no cover
+            elif name.isidentifier():
+                return values
+            else:  # pragma: no cover
                 alias = name
                 name = re.sub(r'\W', '_', name)
-            else:
-                return values
             if not values.get('alias'):
                 values['alias'] = alias
             values['name'] = name
@@ -86,14 +97,17 @@ class DataModelFieldBase(BaseModel):
         return repr(self.default)
 
 
+@lru_cache()
+def get_template(template_file_path: Path) -> Template:
+    loader = FileSystemLoader(str(TEMPLATE_DIR / template_file_path.parent))
+    environment: Environment = Environment(loader=loader)
+    return environment.get_template(template_file_path.name)
+
+
 class TemplateBase(ABC):
     def __init__(self, template_file_path: Path) -> None:
         self.template_file_path: Path = template_file_path
-        loader = FileSystemLoader(str(TEMPLATE_DIR / template_file_path.parent))
-        self.environment: Environment = Environment(loader=loader)
-        self._template: Template = self.environment.get_template(
-            template_file_path.name
-        )
+        self._template: Template = get_template(template_file_path)
 
     @property
     def template(self) -> Template:
@@ -133,6 +147,7 @@ class DataModel(TemplateBase, ABC):
         methods: Optional[List[str]] = None,
         path: Optional[Path] = None,
         description: Optional[str] = None,
+        reference: Optional[Reference] = None,
     ) -> None:
         if not self.TEMPLATE_FILE_PATH:
             raise Exception('TEMPLATE_FILE_PATH is undefined')
@@ -151,6 +166,10 @@ class DataModel(TemplateBase, ABC):
         base_classes = [base_class for base_class in base_classes or [] if base_class]
         self.base_classes: List[str] = base_classes
         self.path: Optional[Path] = path
+        self.reference: Optional[Reference] = reference
+
+        if self.reference:
+            self.reference.source = self
 
         self.reference_classes: Set[str] = {
             r for r in base_classes if r != self.BASE_CLASS
@@ -212,6 +231,11 @@ class DataModel(TemplateBase, ABC):
             ]
         return self.name.split('.')[:-1]
 
+    @property
+    def all_data_types(self) -> Iterator['DataType']:
+        for field in self.fields:
+            yield from field.data_type.all_data_types
+
     def render(self) -> str:
         response = self._render(
             class_name=self.class_name,
