@@ -11,6 +11,7 @@ from datamodel_code_generator.model import (
     DataModelFieldBase,
 )
 from datamodel_code_generator.model.pydantic.imports import IMPORT_EXTRA, IMPORT_FIELD
+from datamodel_code_generator.reference import Reference
 
 
 class Constraints(ConstraintsBase):
@@ -63,6 +64,8 @@ class DataModelField(DataModelFieldBase):
             f"{k}={repr(v)}" for k, v in data.items() if v is not None
         )
         if not field_arguments:
+            if self.nullable and self.required:
+                return 'Field(...)'  # Field() is for mypy
             return ""
 
         value_arg = "..." if self.required else repr(self.default)
@@ -88,6 +91,7 @@ class BaseModel(DataModel):
         imports: Optional[List[Import]] = None,
         path: Optional[Path] = None,
         description: Optional[str] = None,
+        reference: Optional[Reference] = None,
     ):
 
         methods: List[str] = [field.method for field in fields if field.method]
@@ -106,6 +110,7 @@ class BaseModel(DataModel):
             methods=methods,
             path=path,
             description=description,
+            reference=reference,
         )
 
         config_parameters: Dict[str, Any] = {}
