@@ -28,6 +28,7 @@ from ..imports import IMPORT_ANNOTATIONS, Import, Imports
 from ..model import pydantic as pydantic_model
 from ..model.base import ALL_MODEL, DataModel, DataModelFieldBase
 from ..model.enum import Enum
+from ..model.pydantic.imports import IMPORT_FIELD
 from ..reference import ModelResolver
 from ..types import DataType, DataTypeManager
 from . import LiteralType
@@ -54,16 +55,18 @@ def to_hashable(item: Any) -> Any:
     return item
 
 
-def snakify_field(field: DataModelFieldBase) -> None:
+def snakify_field(field: DataModelFieldBase, model: DataModel) -> None:
     if not field.name:
         return
     original_name = field.name
     field.name = camel_to_snake(original_name)
     if field.name != original_name:
         field.alias = original_name
+        if IMPORT_FIELD not in model.fields:  # pragma: no cover
+            model.imports.append(IMPORT_FIELD)
 
 
-def set_strip_default_none(field: DataModelFieldBase) -> None:
+def set_strip_default_none(field: DataModelFieldBase, _: DataModel) -> None:
     field.strip_default_none = True
 
 
@@ -234,6 +237,8 @@ class Parser(ABC):
         reuse_model: bool = False,
         encoding: str = 'utf-8',
         enum_field_as_literal: Optional[LiteralType] = None,
+        set_default_enum_member: bool = False,
+        strict_nullable: bool = False,
     ):
         self.data_type_manager: DataTypeManager = data_type_manager_type(
             target_python_version, use_standard_collections
@@ -258,6 +263,8 @@ class Parser(ABC):
         self.reuse_model: bool = reuse_model
         self.encoding: str = encoding
         self.enum_field_as_literal: Optional[LiteralType] = enum_field_as_literal
+        self.set_default_enum_member: bool = set_default_enum_member
+        self.strict_nullable: bool = strict_nullable
 
         self.current_source_path: Optional[Path] = None
 
@@ -280,7 +287,9 @@ class Parser(ABC):
             self.extra_template_data[ALL_MODEL]['allow_population_by_field_name'] = True
 
         self.model_resolver = ModelResolver(aliases=aliases)
-        self.field_preprocessors: List[Callable[[DataModelFieldBase], None]] = []
+        self.field_preprocessors: List[
+            Callable[[DataModelFieldBase, DataModel], None]
+        ] = []
         if self.snake_case_field:
             self.field_preprocessors.append(snakify_field)
         if self.strip_default_none:
@@ -305,7 +314,7 @@ class Parser(ABC):
     def append_result(self, data_model: DataModel) -> None:
         for field_preprocessor in self.field_preprocessors:
             for field in data_model.fields:
-                field_preprocessor(field)
+                field_preprocessor(field, data_model)
         self.results.append(data_model)
 
     @property
@@ -374,8 +383,8 @@ class Parser(ABC):
             import_map: Dict[str, Tuple[str, str]] = {}
             model_names: Set[str] = {m.name for m in models}
             processed_models: Set[str] = set()
-            model_cache: Dict[Tuple[str, ...], str] = {}
-            duplicated_model_names: Dict[str, str] = {}
+            model_cache: Dict[Tuple[str, ...], DataModel] = {}
+            duplicated_models: Dict[str, DataModel] = {}
             for model in models:
                 alias_map: Dict[str, Optional[str]] = {}
                 if model.name in require_update_action_models:
@@ -388,25 +397,30 @@ class Parser(ABC):
                         models_to_update += [model.name]
                         processed_models.add(model.name)
                 imports.append(model.imports)
+                model.reference_classes = {
+                    r for r in model.reference_classes if r not in model_names
+                }
                 for data_type in model.all_data_types:
-                    if not data_type.is_modular:
+                    # To change from/import
+
+                    if not data_type.reference:
+                        # No need to import non-reference model.
                         continue
-                    full_name = data_type.full_name
-                    from_, import_ = relative(module_path, full_name)
+
+                    if data_type.reference.source in models:
+                        # Referenced model is in the same file. we don't need to import the model
+                        data_type.type = data_type.name
+                        continue
+                    elif (
+                        isinstance(self.source, Path)
+                        and self.source.is_file()
+                        and self.source.name == data_type.reference.path.split('#/')[0]
+                    ):
+                        full_name = data_type.type
+                    else:
+                        full_name = data_type.full_name
+                    from_, import_ = relative(module_path, full_name)  # type: ignore
                     name = data_type.name
-                    if data_type.reference:
-                        reference = self.model_resolver.get(data_type.reference.path)
-                        if reference and (
-                            (
-                                isinstance(self.source, Path)
-                                and self.source.is_file()
-                                and self.source.name == reference.path.split('#/')[0]
-                            )
-                            or reference.actual_module_name == module_path
-                        ):
-                            if name in model.reference_classes:  # pragma: no cover
-                                model.reference_classes.remove(name)
-                            continue
                     full_path = f'{from_}/{import_}'
                     if full_path in alias_map:
                         alias = alias_map[full_path] or import_
@@ -415,21 +429,28 @@ class Parser(ABC):
                             full_path.split('/'), import_, unique=True
                         ).name
                         alias_map[full_path] = None if alias == import_ else alias
-                    new_name = f'{alias}.{name}' if from_ and import_ else name
-                    if data_type.module_name and not full_name.startswith(from_):
-                        import_map[new_name] = (
-                            f'.{full_name[:len(new_name) * - 1 - 1]}',
-                            new_name.split('.')[0],
-                        )
+                    new_name = (
+                        f'{alias}.{name}'
+                        if (from_ and import_ and alias != name)
+                        else name
+                    )
                     if name in model.reference_classes:
                         model.reference_classes.remove(name)
                         model.reference_classes.add(new_name)
                     data_type.type = new_name
 
+                    if (
+                        full_name
+                        and full_name.startswith(from_)
+                        or f'.{full_name}'.startswith(from_)
+                    ):
+                        import_map[new_name] = (
+                            f'.{full_name[:len(new_name) * - 1 - 1]}',  # type: ignore
+                            new_name.split('.')[0],
+                        )
+
                 for ref_name in model.reference_classes:
-                    if ref_name in model_names:
-                        continue
-                    elif ref_name in import_map:
+                    if ref_name in import_map:
                         from_, import_ = import_map[ref_name]
                     else:
                         from_, import_ = relative(module_path, ref_name)
@@ -452,34 +473,44 @@ class Parser(ABC):
                             model.fields,
                         )
                     )
-                    cached_model_name = model_cache.get(model_key)
-                    if cached_model_name:
+                    cached_model = model_cache.get(model_key)
+                    if cached_model:
                         if isinstance(model, Enum):
-                            duplicated_model_names[model.name] = cached_model_name
+                            duplicated_models[model.name] = cached_model
                         else:
                             index = models.index(model)
                             inherited_model = model.__class__(
                                 name=model.name,
                                 fields=[],
-                                base_classes=[cached_model_name],
+                                base_classes=[cached_model.name],
                                 description=model.description,
                             )
                             models.insert(index, inherited_model)
                             models.remove(model)
 
                     else:
-                        model_cache[model_key] = model.name
+                        model_cache[model_key] = model
 
             if self.reuse_model:
                 for model in models:
                     for data_type in model.all_data_types:  # pragma: no cover
                         if data_type.type:
-                            duplicated_model_name = duplicated_model_names.get(
-                                data_type.type
-                            )
-                            if duplicated_model_name:
-                                data_type.type = duplicated_model_name
-
+                            duplicated_model = duplicated_models.get(data_type.type)
+                            if duplicated_model:
+                                data_type.type = duplicated_model.name
+                                data_type.reference = duplicated_model.reference
+            if self.set_default_enum_member:
+                for model in models:  # pragma: no cover
+                    for model_field in model.fields:
+                        if model_field.default:
+                            for data_type in model_field.data_type.all_data_types:
+                                if data_type.reference:
+                                    if isinstance(data_type.reference.source, Enum):
+                                        enum_member = data_type.reference.source.find_member(
+                                            model_field.default
+                                        )
+                                        if enum_member:
+                                            model_field.default = enum_member
             if with_import:
                 result += [str(imports), str(self.imports), '\n']
 
