@@ -32,7 +32,6 @@ from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model.enum import Enum
 from datamodel_code_generator.parser import LiteralType
 
-from ..imports import IMPORT_LITERAL, Import
 from ..model import pydantic as pydantic_model
 from ..parser.base import Parser
 from ..reference import is_url
@@ -129,6 +128,14 @@ class JsonSchemaObject(BaseModel):
             del values['exclusiveMinimum']
         return values
 
+    @validator('ref')
+    def validate_ref(cls, value: Any) -> Any:
+        if isinstance(value, str) and '#' in value:
+            if '#/' in value or value[0] == '#' or value[-1] == '#':
+                return value
+            return value.replace('#', '#/')
+        return value
+
     items: Union[List['JsonSchemaObject'], 'JsonSchemaObject', None]
     uniqueItem: Optional[bool]
     type: Union[str, List[str], None]
@@ -167,7 +174,13 @@ class JsonSchemaObject(BaseModel):
 
     @cached_property
     def is_object(self) -> bool:
-        return self.properties is not None or self.type == 'object'
+        return (
+            self.properties is not None
+            or self.type == 'object'
+            and not self.allOf
+            and not self.oneOf
+            and not self.anyOf
+        )
 
     @cached_property
     def is_array(self) -> bool:
@@ -234,6 +247,8 @@ class JsonSchemaParser(Parser):
         reuse_model: bool = False,
         encoding: str = 'utf-8',
         enum_field_as_literal: Optional[LiteralType] = None,
+        set_default_enum_member: bool = False,
+        strict_nullable: bool = False,
     ):
         super().__init__(
             source=source,
@@ -261,6 +276,8 @@ class JsonSchemaParser(Parser):
             reuse_model=reuse_model,
             encoding=encoding,
             enum_field_as_literal=enum_field_as_literal,
+            set_default_enum_member=set_default_enum_member,
+            strict_nullable=strict_nullable,
         )
 
         self.remote_object_cache: Dict[str, Dict[str, Any]] = {}
@@ -355,14 +372,14 @@ class JsonSchemaParser(Parser):
                         data_types=[self.get_data_type(item.items)], is_list=True,
                     )
                 )
-            else:
+            elif item.is_object:
                 data_types.append(
-                    self.data_type.from_model_name(
-                        self.parse_object(
-                            name, item, [*path, str(index)], singular_name=True,
-                        ).name,
+                    self.parse_object(
+                        name, item, [*path, str(index)], singular_name=True,
                     )
                 )
+            else:
+                data_types.append(self.get_data_type(item))
         return self.data_type(data_types=data_types)
 
     def parse_any_of(
@@ -392,9 +409,10 @@ class JsonSchemaParser(Parser):
                 base_classes.append(self.get_ref_data_type(all_of_item.ref))
             else:
                 fields.extend(self.parse_object_fields(all_of_item, path))
-        class_name = self.model_resolver.add(
+        reference = self.model_resolver.add(
             path, name, class_name=True, unique=True, loaded=True
-        ).name
+        )
+        class_name = reference.name
         self.set_additional_properties(class_name, obj)
         data_model_type = self.data_model_type(
             class_name,
@@ -406,13 +424,14 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            reference=reference,
         )
         # add imports for the fields
         for f in fields:
             data_model_type.imports.extend(f.imports)
         self.append_result(data_model_type)
 
-        return self.data_type.from_model_name(class_name)
+        return self.data_type.from_reference(reference)
 
     def parse_object_fields(
         self, obj: JsonSchemaObject, path: List[str]
@@ -424,7 +443,6 @@ class JsonSchemaParser(Parser):
         fields: List[DataModelFieldBase] = []
 
         for field_name, field in properties.items():
-            field_type: DataType
             original_field_name: str = field_name
             constraints: Optional[Mapping[str, Any]] = None
             field_name, alias = self.model_resolver.get_valid_field_name_and_alias(
@@ -433,10 +451,9 @@ class JsonSchemaParser(Parser):
             if field.ref:
                 field_type = self.get_ref_data_type(field.ref)
             elif field.is_array:
-                array_field = self.parse_array_fields(
+                field_type = self.parse_array_fields(
                     field_name, field, [*path, field_name]
-                )
-                field_type = array_field.data_type
+                ).data_type
                 constraints = field.dict()
             elif field.anyOf:
                 field_type = self.parse_any_of(field_name, field, [*path, field_name])
@@ -446,13 +463,10 @@ class JsonSchemaParser(Parser):
                 field_type = self.parse_all_of(field_name, field, [*path, field_name])
             elif field.is_object:
                 if field.properties:
-                    field_type = self.data_type.from_model_name(
-                        self.parse_object(field_name, field, [*path, field_name]).name
+                    field_type = self.parse_object(
+                        field_name, field, [*path, field_name]
                     )
-
                 elif isinstance(field.additionalProperties, JsonSchemaObject):
-                    unresolved_type: Optional[str]
-                    imports_: List[Import] = []
                     field_class_name = self.model_resolver.add(
                         [*path, field_name], field_name, class_name=True
                     ).name
@@ -462,70 +476,56 @@ class JsonSchemaParser(Parser):
                         isinstance(field.additionalProperties.items, JsonSchemaObject)
                         and field.additionalProperties.items.ref
                     ):
-                        unresolved_type = self.model_resolver.add_ref(
-                            field.additionalProperties.items.ref,
-                        ).name
-                        additional_properties_type = self.data_type.from_model_name(
-                            unresolved_type, is_list=True
-                        ).type_hint
+                        additional_properties_type = self.data_type.from_reference(
+                            self.model_resolver.add_ref(
+                                field.additionalProperties.items.ref,
+                            ),
+                            is_list=True,
+                        )
                     elif field.additionalProperties.is_array:
-                        additional_properties_type = unresolved_type = self.parse_array(
+                        additional_properties_type = self.parse_array(
                             field_class_name,
                             field.additionalProperties,
                             [*path, field_name],
-                        ).name
+                        )
                     elif field.additionalProperties.is_object:
-                        additional_properties_type = (
-                            unresolved_type
-                        ) = self.parse_object(
+                        additional_properties_type = self.parse_object(
                             field_class_name,
                             field.additionalProperties,
                             [*path, field_name],
                             additional_properties=None
                             if field.additionalProperties.ref
                             else field,
-                        ).name
+                        )
                     elif field.additionalProperties.enum:
                         if self.should_parse_enum_as_literal(
                             field.additionalProperties
                         ):
                             additional_properties_type = self.data_type.create_literal(
                                 field.additionalProperties.enum
-                            ).type_hint
-                            unresolved_type = None
-                            imports_.append(IMPORT_LITERAL)
+                            )
                         else:
-                            additional_properties_type = (
-                                unresolved_type
-                            ) = self.parse_enum(
+                            additional_properties_type = self.parse_enum(
                                 field_class_name,
                                 field.additionalProperties,
                                 [*path, field_name],
-                            ).name
+                            )
 
                     else:
-                        additional_properties_type = (
-                            unresolved_type
-                        ) = self.parse_root_type(
+                        additional_properties_type = self.parse_root_type(
                             field_class_name,
                             field.additionalProperties,
                             [*path, field_name],
                             additional_properties=None
                             if field.additionalProperties.ref
                             else field,
-                        ).name
+                        )
                     self.parse_ref(
                         field.additionalProperties, [*path, field_name],
                     )
                     field_type = self.data_type(
-                        type=additional_properties_type,
-                        is_dict=True,
-                        unresolved_types={unresolved_type}
-                        if unresolved_type
-                        else {*()},
-                        imports_=imports_,
+                        data_types=[additional_properties_type], is_dict=True
                     )
-
                 else:
                     field_type = self.data_type_manager.get_data_type(Types.object)
             elif field.enum:
@@ -534,8 +534,7 @@ class JsonSchemaParser(Parser):
                         field.enum
                     )
                 else:
-                    enum = self.parse_enum(field_name, field, [*path, field_name])
-                    field_type = self.data_type.from_model_name(enum.name)
+                    field_type = self.parse_enum(field_name, field, [*path, field_name])
             else:
                 field_type = self.get_data_type(field)
                 if self.field_constraints:
@@ -558,6 +557,7 @@ class JsonSchemaParser(Parser):
                     required=required,
                     alias=alias,
                     constraints=constraints,
+                    nullable=field.nullable if self.strict_nullable else None,
                 )
             )
         return fields
@@ -570,21 +570,22 @@ class JsonSchemaParser(Parser):
         singular_name: bool = False,
         unique: bool = True,
         additional_properties: Optional[JsonSchemaObject] = None,
-    ) -> DataModel:
+    ) -> DataType:
         if not unique:  # pragma: no cover
             warn(
                 f'{self.__class__.__name__}.parse_object() ignore `unique` argument.'
                 f'An object name must be unique.'
                 f'This argument will be removed in a future version'
             )
-        class_name = self.model_resolver.add(
+        reference = self.model_resolver.add(
             path,
             name,
             class_name=True,
             singular_name=singular_name,
             unique=True,
             loaded=True,
-        ).name
+        )
+        class_name = reference.name
         self.set_title(class_name, obj)
         self.set_additional_properties(class_name, additional_properties or obj)
         data_model_type = self.data_model_type(
@@ -595,9 +596,10 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            reference=reference,
         )
         self.append_result(data_model_type)
-        return data_model_type
+        return self.data_type.from_reference(reference)
 
     def parse_array_fields(
         self, name: str, obj: JsonSchemaObject, path: List[str]
@@ -610,23 +612,24 @@ class JsonSchemaParser(Parser):
         for index, item in enumerate(items):
             field_path = [*path, str(index)]
             if item.has_constraint:
-                model = self.model_resolver.add(
-                    field_path, name, class_name=True, singular_name=True, unique=True,
-                )
                 item_obj_data_types.append(
-                    self.data_type.from_model_name(
-                        self.parse_root_type(model.name, item, field_path,).name
+                    self.parse_root_type(
+                        self.model_resolver.add(
+                            field_path,
+                            name,
+                            class_name=True,
+                            singular_name=True,
+                            unique=True,
+                        ).name,
+                        item,
+                        field_path,
                     )
                 )
             elif item.ref:
                 item_obj_data_types.append(self.get_ref_data_type(item.ref))
             elif isinstance(item, JsonSchemaObject) and item.properties:
                 item_obj_data_types.append(
-                    self.data_type.from_model_name(
-                        self.parse_object(
-                            name, item, field_path, singular_name=True,
-                        ).name,
-                    )
+                    self.parse_object(name, item, field_path, singular_name=True,)
                 )
             elif item.anyOf:
                 item_obj_data_types.append(self.parse_any_of(name, item, field_path))
@@ -641,31 +644,36 @@ class JsonSchemaParser(Parser):
                     )
                 )
             elif item.enum:
-                item_obj_data_types.append(
-                    self.data_type_manager.data_type.create_literal(item.enum)
-                    if self.should_parse_enum_as_literal(item)
-                    else self.data_type.from_model_name(
-                        self.parse_enum(
-                            name, item, field_path, singular_name=True
-                        ).name,
+                if self.should_parse_enum_as_literal(item):
+                    item_obj_data_types.append(
+                        self.data_type_manager.data_type.create_literal(item.enum)
+                    )
+                else:
+                    item_obj_data_types.append(
+                        self.parse_enum(name, item, field_path, singular_name=True)
                     )
-                )
             elif item.is_array:
-                array_field = self.parse_array_fields(
-                    self.model_resolver.add(field_path, name, class_name=True).name,
-                    item,
-                    field_path,
+                item_obj_data_types.append(
+                    self.parse_array_fields(
+                        self.model_resolver.add(field_path, name, class_name=True).name,
+                        item,
+                        field_path,
+                    ).data_type
                 )
-                item_obj_data_types.append(array_field.data_type)
             else:
                 item_obj_data_types.append(self.get_data_type(item))
         if self.force_optional_for_required_fields:
             required: bool = False
         else:
-            required = not obj.nullable and not (
-                obj.has_default and self.apply_default_values_for_required_fields
-            )
-        field = self.data_model_field_type(
+            if self.strict_nullable:
+                required = not (
+                    obj.has_default and self.apply_default_values_for_required_fields
+                )
+            else:
+                required = not obj.nullable and not (
+                    obj.has_default and self.apply_default_values_for_required_fields
+                )
+        return self.data_model_field_type(
             data_type=self.data_type(data_types=item_obj_data_types, is_list=True,),
             example=obj.example,
             examples=obj.examples,
@@ -674,14 +682,14 @@ class JsonSchemaParser(Parser):
             title=obj.title,
             required=required,
             constraints=obj.dict(),
+            nullable=obj.nullable if self.strict_nullable else None,
         )
-        return field
 
     def parse_array(
         self, name: str, obj: JsonSchemaObject, path: List[str]
-    ) -> DataModel:
+    ) -> DataType:
         field = self.parse_array_fields(name, obj, [*path, name])
-        self.model_resolver.add(path, name, loaded=True)
+        reference = self.model_resolver.add(path, name, loaded=True)
         data_model_root = self.data_model_root_type(
             name,
             [field],
@@ -690,10 +698,10 @@ class JsonSchemaParser(Parser):
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            reference=reference,
         )
-
         self.append_result(data_model_root)
-        return data_model_root
+        return self.data_type.from_reference(reference)
 
     def parse_root_type(
         self,
@@ -701,7 +709,7 @@ class JsonSchemaParser(Parser):
         obj: JsonSchemaObject,
         path: List[str],
         additional_properties: Optional[JsonSchemaObject] = None,
-    ) -> DataModel:
+    ) -> DataType:
         if obj.type:
             data_type: DataType = self.get_data_type(obj)
         elif obj.anyOf:
@@ -713,10 +721,15 @@ class JsonSchemaParser(Parser):
         if self.force_optional_for_required_fields:
             required: bool = False
         else:
-            required = not obj.nullable and not (
-                obj.has_default and self.apply_default_values_for_required_fields
-            )
-        self.model_resolver.add(path, name, loaded=True)
+            if self.strict_nullable:
+                required = not (
+                    obj.has_default and self.apply_default_values_for_required_fields
+                )
+            else:
+                required = not obj.nullable and not (
+                    obj.has_default and self.apply_default_values_for_required_fields
+                )
+        reference = self.model_resolver.add(path, name, loaded=True)
         self.set_title(name, obj)
         self.set_additional_properties(name, additional_properties or obj)
         data_model_root_type = self.data_model_root_type(
@@ -730,15 +743,17 @@ class JsonSchemaParser(Parser):
                     default=obj.default,
                     required=required,
                     constraints=obj.dict() if self.field_constraints else {},
+                    nullable=obj.nullable if self.strict_nullable else None,
                 )
             ],
             custom_base_class=self.base_class,
             custom_template_dir=self.custom_template_dir,
             extra_template_data=self.extra_template_data,
             path=self.current_source_path,
+            reference=reference,
         )
         self.append_result(data_model_root_type)
-        return data_model_root_type
+        return self.data_type.from_reference(reference)
 
     def parse_enum(
         self,
@@ -747,7 +762,7 @@ class JsonSchemaParser(Parser):
         path: List[str],
         singular_name: bool = False,
         unique: bool = True,
-    ) -> DataModel:
+    ) -> DataType:
         if not unique:  # pragma: no cover
             warn(
                 f'{self.__class__.__name__}.parse_enum() ignore `unique` argument.'
@@ -779,7 +794,7 @@ class JsonSchemaParser(Parser):
                     required=True,
                 )
             )
-        enum_name = self.model_resolver.add(
+        reference = self.model_resolver.add(
             path,
             name,
             class_name=True,
@@ -787,15 +802,16 @@ class JsonSchemaParser(Parser):
             singular_name_suffix='Enum',
             unique=True,
             loaded=True,
-        ).name
+        )
         enum = Enum(
-            enum_name,
+            reference.name,
             fields=enum_fields,
             path=self.current_source_path,
             description=obj.description if self.use_schema_description else None,
+            reference=reference,
         )
         self.append_result(enum)
-        return enum
+        return self.data_type.from_reference(reference)
 
     def _get_ref_body(self, ref: str) -> Dict[Any, Any]:
         if is_url(ref):
@@ -844,7 +860,7 @@ class JsonSchemaParser(Parser):
 
     def parse_ref(self, obj: JsonSchemaObject, path: List[str]) -> None:
         if obj.ref:
-            reference = self.model_resolver.get(obj.ref)
+            reference = self.model_resolver.add_ref(obj.ref)
             if not reference or not reference.loaded:
                 # https://swagger.io/docs/specification/using-ref/
                 if obj.ref_type == JSONReference.LOCAL:
@@ -863,15 +879,9 @@ class JsonSchemaParser(Parser):
                         relative_path, object_path = ref[:-1], ''
                     else:
                         relative_path, object_path = ref, ''
-                    object_paths = object_path.split('/')
-                    ref_body = self._get_ref_body(relative_path)
 
-                    if object_path:
-                        models = get_model_by_path(ref_body, object_paths)
-                        model_name = object_paths[-1]
-                    else:
-                        models = ref_body
-                        model_name = self.model_resolver.add_ref(obj.ref).name
+                    models = self._get_ref_body(relative_path)
+                    model_name = self.model_resolver.add_ref(obj.ref).name
                     if obj.ref_type == JSONReference.REMOTE:
                         previous_base_path: Optional[Path] = self.base_path
                         self.base_path = (self.base_path / relative_path).parent
@@ -881,8 +891,8 @@ class JsonSchemaParser(Parser):
                     self._parse_file(
                         models,
                         model_name,
-                        [*relative_paths, '#', *object_paths],
                         relative_paths,
+                        object_path.split('/') if object_path else None,
                     )
                     if previous_base_path:
                         self.base_path = previous_base_path
@@ -917,12 +927,12 @@ class JsonSchemaParser(Parser):
             else:
                 for item in obj.items:
                     self.parse_id(item, path)
-        if isinstance(obj.additionalProperties, JsonSchemaObject):  # pragma: no cover
+        if isinstance(obj.additionalProperties, JsonSchemaObject):
             self.parse_id(obj.additionalProperties, path)
-        for item in obj.anyOf:  # pragma: no cover
-            self.parse_id(item, path)  # pragma: no cover
+        for item in obj.anyOf:
+            self.parse_id(item, path)
         for item in obj.allOf:
-            self.parse_id(item, path)  # pragma: no cover
+            self.parse_id(item, path)
         if obj.properties:
             for value in obj.properties.values():
                 self.parse_id(value, path)
@@ -983,7 +993,7 @@ class JsonSchemaParser(Parser):
                     obj_name = self.raw_obj.get('title', 'Model')
                     if not self.model_resolver.validate_name(obj_name):
                         raise InvalidClassNameError(obj_name)
-                self._parse_file(self.raw_obj, obj_name, [*path_parts], path_parts)
+                self._parse_file(self.raw_obj, obj_name, path_parts)
 
         # resolve unparsed json pointer
         for source in self.iter_source:
@@ -1003,7 +1013,7 @@ class JsonSchemaParser(Parser):
                     if (
                         len(valid_path_parts) == 1
                         and self.model_resolver.add_ref(f'{file_path}#').loaded
-                    ):  # pragma: no cover
+                    ):
                         continue
                     # for root model
                     self.raw_obj = load_yaml(source.text)
@@ -1028,10 +1038,17 @@ class JsonSchemaParser(Parser):
         raw: Dict[str, Any],
         obj_name: str,
         path_parts: List[str],
-        root_path: List[str],
+        object_paths: Optional[List[str]] = None,
     ) -> None:
-        with self.model_resolver.current_root_context(root_path):
-            obj_name = self.model_resolver.add(path_parts, obj_name, unique=False).name
+        if object_paths:
+            path = [*path_parts, f'#/{object_paths[0]}', *object_paths[1:]]
+        else:
+            path = path_parts
+        reference = self.model_resolver.get(path)
+        if reference and reference.loaded:
+            return
+        with self.model_resolver.current_root_context(path_parts):
+            obj_name = self.model_resolver.add(path, obj_name, unique=False).name
             with self.root_id_context(raw):
 
                 # parse $id before parsing $ref
@@ -1042,6 +1059,14 @@ class JsonSchemaParser(Parser):
                     obj = JsonSchemaObject.parse_obj(model)
                     self.parse_id(obj, [*path_parts, '#/definitions', key])
 
-                self.parse_obj(obj_name, root_obj, path_parts)
+                if object_paths:
+                    models = get_model_by_path(raw, object_paths)
+                    model_name = object_paths[-1]
+                    self.parse_obj(model_name, JsonSchemaObject.parse_obj(models), path)
+                else:
+                    self.parse_obj(obj_name, root_obj, path_parts or ['#'])
                 for key, model in definitions.items():
-                    self.parse_raw_obj(key, model, [*path_parts, '#/definitions', key])
+                    path = [*path_parts, '#/definitions', key]
+                    reference = self.model_resolver.get(path)
+                    if not reference or not reference.loaded:
+                        self.parse_raw_obj(key, model, path)
